type texture =
    Color of Gfx.color
  | Image of Gfx.surface

let white = Color (Gfx.color 255 255 255 2555)


type config = {
  (* Informations de touches *)
  key_left: string;
  key_up : string;
  key_down : string;
  key_right : string;

  (* Informations de fenetre *)
  window : Gfx.window;
  window_surface : Gfx.surface;
  ctx : Gfx.context;

  (* Question 4.2.4 *)
  (* Informations de textures *)
  mutable textures : texture array;
  mutable current : int;

  (* Information de temps *)
  mutable last_dt : float;

  (* Question 4.2.5 *)
  (* Informations de position *)
  mutable x : int;
  mutable y : int;

}

(* On crée une fenêtre *)

let draw_rect config texture x y w h =
  (* Question 4.2.2 *)
  let {ctx; window_surface; _ } = config in
  match texture with
    Color col ->
    Gfx.set_color ctx col;
    Gfx.fill_rect ctx window_surface x y w h
  | Image img ->
    (* Question 4.3.2 *)
    Gfx.blit_scale ctx window_surface img x y w h


let keys : (string, unit) Hashtbl.t = Hashtbl.create 16

let update cfg dt =
  (* Question 4.2.3 *)
  (*
  draw_rect cfg white 0 0 800 600;
  draw_rect cfg (Color (Gfx.color 0 0 0 255)) 100 100 200 200;
  None *)

  (* Question 4.2.5 *)
  let () =
    match Gfx.poll_event () with
      Gfx.NoEvent -> ()
    | Gfx.KeyDown s -> Gfx.debug "%s@\n%!" s; Hashtbl.replace keys s ()
    | Gfx.KeyUp s -> Hashtbl.remove keys s
    | _ -> ()
  in
  if Hashtbl.mem keys cfg.key_left then cfg.x <- cfg.x - 10;
  if Hashtbl.mem keys cfg.key_right then cfg.x <- cfg.x + 10;
  if Hashtbl.mem keys cfg.key_up then cfg.y <- cfg.y - 10;
  if Hashtbl.mem keys cfg.key_down then cfg.y <- cfg.y + 10;




  (* Question 4.2.4 *)
  if dt -. cfg.last_dt > 1000.0 then begin
    Gfx.debug "%f@\n%!" dt;
    cfg.current <-  (cfg.current + 1) mod Array.length cfg.textures;
    cfg.last_dt <- dt
  end;
  draw_rect cfg white 0 0 800 600;
  (* Pour 4.2.4, coordonnées fixes *)
  (* draw_rect cfg cfg.textures.(cfg.current) 100 100 200 200; *)
  (* Pour 4.2.5, on utilise cfg.x, cfg.y *)
  draw_rect cfg cfg.textures.(cfg.current) cfg.x cfg.y 200 200;
  Gfx.commit cfg.ctx;

  (* Pour 4.2.4 renvoyer None, pour 4.2.5 on teste la touche q*)

  if (Hashtbl.mem keys "q") then Some () else None



let run keys =
  (* Question 4.2.1 *)
  let window = Gfx.create "game_canvas:800x600:" in
  let window_surface = Gfx.get_surface window in
  let ctx = Gfx.get_context window in

  (* Question 4.3.1 *)
  let tile_set_r = Gfx.load_file "resources/files/tile_set.txt" in
  Gfx.main_loop
    (fun _dt -> Gfx.get_resource_opt tile_set_r)
    (fun txt ->
       let images_r =
         txt
         |> String.split_on_char '\n'
         |> List.filter (fun s -> s <> "") (* retire les lignes vides *)
         |> List.map (fun s -> Gfx.load_image ctx ("resources/images/" ^ s))
       in
       Gfx.main_loop (fun _dt ->
           if List.for_all Gfx.resource_ready images_r then
             Some (List.map Gfx.get_resource images_r)
           else None
         )
         (fun images ->
            let textures = images
                           |> List.map (fun img -> Image img)
                           |> Array.of_list
            in


            let cfg = {
              (* Question 4.2.3 *)
              key_left = keys.(0);
              key_right = keys.(1);
              key_up = keys.(2);
              key_down = keys.(3);
              window;
              window_surface;
              ctx;
              (* Question 4.2.4 *)
              last_dt = 0.0;
              textures; (* réutilise textures défini plus haut *)
              current = 0;
              (* Question 4.2.5 *)
              x = 100;
              y = 100;
            }
            in Gfx.main_loop (update cfg) (fun () -> ())
         ))
